#!/usr/bin/env python
import os
import copy
import json
import unittest
import manifest

from emodpy.utils import EradicationBambooBuilds
from emodpy.bamboo import get_model_files
import emod_api.campaign as camp

current_directory = os.path.dirname(os.path.realpath(__file__))


class TestIntervention(unittest.TestCase):
    """
    Base Test class for TBHIV intervention
    """
    @classmethod
    def setUpClass(cls) -> None:
        cls.output = os.path.join(current_directory, 'output')
        if not os.path.isdir(cls.output):
            os.mkdir(cls.output)

        cls.schema = manifest.schema_file
        if not os.path.isfile(cls.schema):
            plan = EradicationBambooBuilds.TBHIV
            print("Retrieving Eradication and schema.json from Bamboo...")
            get_model_files(plan, manifest)

    def setUp(self) -> None:
        print(f"\n{self._testMethodName} started...")
        camp.pubsub_signals_pubbing = []
        camp.pubsub_signals_subbing = []

    def trigger_treatment_list_test(self, intervention_fn, intervention_class_name,
                                    campaign_filename="camp.json", is_multi=True):
        camp.set_schema( self.schema )
        expected_trigger_treatment_list = ["NewInfection"]
        trigger_treatment_list = copy.deepcopy(expected_trigger_treatment_list)
        print( f"Calling intervention function with signal list {trigger_treatment_list}." )
        event = intervention_fn(camp, trigger_treatment_list)
        camp.add(event, first=True)
        camp_filename = os.path.join(self.output, campaign_filename)
        manifest.delete_existing_file(camp_filename)
        camp.save(camp_filename)
        self.assertTrue(os.path.isfile(camp_filename))

        with open(camp_filename, 'r') as file:
            camp_file = json.load(file)
        intervention_config = camp_file["Events"][0]["Event_Coordinator_Config"]["Intervention_Config"]
        self.assertEqual(intervention_config["Trigger_Condition_List"],
                         expected_trigger_treatment_list)
        self.assertEqual(camp.pubsub_signals_subbing,
                         copy.deepcopy(expected_trigger_treatment_list))
        if not is_multi:
            self.assertEqual(intervention_config["Actual_IndividualIntervention_Config"]["class"],
                             intervention_class_name)
        else:
            interventions = intervention_config["Actual_IndividualIntervention_Config"]["Intervention_List"]
            iv_classes = []
            for inter in interventions:
                iv_classes.append(inter["class"])
            self.assertIn(intervention_class_name, iv_classes)

    def trigger_treatment_list_with_drug_name_test(self, intervention_fn, drug_name,
                                                   intervention_class_name,
                                                   campaign_filename="camp.json",
                                                   is_multi=True):
        camp.set_schema( self.schema )
        expected_trigger_treatment_list = ["NewInfection"]
        trigger_treatment_list = copy.deepcopy(expected_trigger_treatment_list)
        event = intervention_fn(camp, trigger_treatment_list, drug_name)
        camp.add(event, first=True)
        camp_filename = os.path.join(self.output, campaign_filename)
        manifest.delete_existing_file(camp_filename)
        camp.save(camp_filename)
        self.assertTrue(os.path.isfile(camp_filename))

        with open(camp_filename, 'r') as file:
            camp_file = json.load(file)
        intervention_config = camp_file["Events"][0]["Event_Coordinator_Config"]["Intervention_Config"]
        self.assertEqual(intervention_config["Trigger_Condition_List"], expected_trigger_treatment_list)
        self.assertEqual(camp.pubsub_signals_subbing,
                         copy.deepcopy(expected_trigger_treatment_list))
        if not is_multi:
            self.assertEqual(intervention_config["Actual_IndividualIntervention_Config"]["class"],
                             intervention_class_name)
            self.assertEqual(intervention_config["Actual_IndividualIntervention_Config"]["TB_Drug_Name"],
                             drug_name)
        else:
            intervention_list = intervention_config["Actual_IndividualIntervention_Config"]["Intervention_List"]
            found_it = False
            for listed_intervention in intervention_list:
                if listed_intervention["class"] == intervention_class_name:
                    found_it = True
                    self.assertEqual(listed_intervention["TB_Drug_Name"], drug_name)
            self.assertTrue(found_it, msg=f"Expected to find {intervention_class_name} in list,"
                                          f" got {str(intervention_list)}")

    def trigger_treatment_list_with_output_event_test(self, intervention_fn, output_event,
                                                      intervention_class_name, campaign_filename="camp.json"):
        camp.set_schema( self.schema )
        expected_trigger_treatment_list = ["NewInfection"]
        trigger_treatment_list = copy.deepcopy(expected_trigger_treatment_list)
        event = intervention_fn(camp, trigger_treatment_list, output_event)
        camp.add(event, first=True)
        camp_filename = os.path.join(self.output, campaign_filename)
        manifest.delete_existing_file(camp_filename)
        camp.save(camp_filename)
        self.assertTrue(os.path.isfile(camp_filename))

        with open(camp_filename, 'r') as file:
            camp_file = json.load(file)
        self.assertEqual(camp_file["Events"][0]["Event_Coordinator_Config"]["Intervention_Config"]
                         ["Trigger_Condition_List"], expected_trigger_treatment_list)
        self.assertEqual(camp.pubsub_signals_subbing,
                         copy.deepcopy(expected_trigger_treatment_list))
        intervention = camp_file["Events"][0]["Event_Coordinator_Config"]["Intervention_Config"]["Actual_IndividualIntervention_Config"]

        self.assertEqual(intervention["class"], intervention_class_name)
        self.assertEqual(intervention["Actual_IndividualIntervention_Event"],
                         output_event)

    def trigger_treatment_list_with_properties_test(self, intervention_fn, property_to_change,
                                                    final_prop_value, intervention_class_name,
                                                    campaign_filename="camp.json"):
        camp.set_schema( self.schema )
        expected_trigger_treatment_list = ["NewInfection"]
        trigger_treatment_list = copy.deepcopy(expected_trigger_treatment_list)
        event = intervention_fn(camp, trigger_treatment_list, property_to_change, final_prop_value)
        camp.add(event, first=True)
        camp_filename = os.path.join(self.output, campaign_filename)
        manifest.delete_existing_file(camp_filename)
        camp.save(camp_filename)
        self.assertTrue(os.path.isfile(camp_filename))

        with open(camp_filename, 'r') as file:
            camp_file = json.load(file)
        self.assertEqual(camp_file["Events"][0]["Event_Coordinator_Config"]["Intervention_Config"]
                         ["Trigger_Condition_List"], expected_trigger_treatment_list)
        self.assertEqual(camp.pubsub_signals_subbing,
                         copy.deepcopy(expected_trigger_treatment_list))
        intervention = camp_file["Events"][0]["Event_Coordinator_Config"]["Intervention_Config"]["Actual_IndividualIntervention_Config"]
        self.assertEqual(intervention["class"], intervention_class_name)
        self.assertEqual(intervention["Target_Property_Key"],
                         property_to_change)
        self.assertEqual(intervention["Target_Property_Value"],
                         final_prop_value)

    def sensitivity_and_specificity_test(self, intervention_fn, base_sensitivity, base_specificity,
                                         campaign_filename):
        camp.set_schema( self.schema )
        expected_trigger_treatment_list = ["NewInfection"]
        trigger_treatment_list = copy.deepcopy(expected_trigger_treatment_list)
        event = intervention_fn(camp, trigger_treatment_list, base_sensitivity=base_sensitivity,
                                base_specificity=base_specificity)
        camp.add(event, first=True)
        camp_filename = os.path.join(self.output, campaign_filename)
        manifest.delete_existing_file(camp_filename)
        camp.save(camp_filename)
        self.assertTrue(os.path.isfile(camp_filename))

        with open(camp_filename, 'r') as file:
            camp_file = json.load(file)
        self.assertEqual(camp_file["Events"][0]["Event_Coordinator_Config"]["Intervention_Config"]
                         ["Actual_IndividualIntervention_Config"]["Base_Sensitivity"], base_sensitivity)
        self.assertEqual(camp_file["Events"][0]["Event_Coordinator_Config"]["Intervention_Config"]
                         ["Actual_IndividualIntervention_Config"]["Base_Specificity"], base_specificity)

    def new_intervention_as_file_test(self, intervention_fn, camp_filename, intervention_class_name, is_multi=True):
        intervention_fn.new_intervention_as_file(camp, filename=camp_filename)
        self.assertTrue(os.path.isfile(camp_filename))

        with open(camp_filename, 'r') as file:
            camp_file = json.load(file)
        ecc = camp_file["Events"][0]["Event_Coordinator_Config"]
        ic = ecc["Intervention_Config"]

        intervention = ic["Actual_IndividualIntervention_Config"]

        if not is_multi:
            self.assertEqual(intervention["class"], intervention_class_name)
        else:
            found_it = False
            interventions = intervention['Intervention_List']
            for inter in interventions:
                if inter["class"] == intervention_class_name:
                    found_it = True
                    self.assertEqual(inter["class"], intervention_class_name)
            self.assertTrue(found_it, msg=f"Expected to find {intervention_class_name} in list,"
                                          f" got {str(intervention['class'])}")
